from __future__ import annotations

import re

import pytest
from yarl import URL

from instawow import results as R
from instawow.common import Flavour, Strategy
from instawow.manager import Manager
from instawow.models import Pkg
from instawow.resolvers import (
    CurseResolver,
    Defn,
    GithubResolver,
    Resolver,
    TukuiResolver,
    WowiResolver,
)
from instawow.utils import evolve_model_obj


@pytest.mark.asyncio
@pytest.mark.parametrize('strategy', [Strategy.default, Strategy.latest])
async def test_curse_simple_strategies(iw_manager: Manager, strategy: Strategy):
    flavourful = Defn('curse', 'molinari', strategy=strategy)
    retail_only = Defn('curse', 'mythic-dungeon-tools', strategy=strategy)
    classic_only = Defn('curse', 'classiccastbars', strategy=strategy)

    results = await iw_manager.resolve([flavourful, retail_only, classic_only])

    if iw_manager.config.game_flavour is Flavour.vanilla_classic:
        assert results[flavourful].version.endswith('Release-classic')
        assert (
            type(results[retail_only]) is R.PkgFileUnavailable
            and results[retail_only].message
            == f"no files matching vanilla_classic using {strategy} strategy"
        )
        assert type(results[classic_only]) is Pkg

    elif iw_manager.config.game_flavour is Flavour.burning_crusade_classic:
        assert results[flavourful].version.endswith('Release-bcc')
        assert (
            type(results[retail_only]) is R.PkgFileUnavailable
            and results[retail_only].message
            == f"no files matching classic using {strategy} strategy"
        )
        assert type(results[classic_only]) is Pkg

    elif iw_manager.config.game_flavour is Flavour.retail:
        assert results[flavourful].version.endswith('Release')
        assert type(results[retail_only]) is Pkg
        assert (
            type(results[classic_only]) is R.PkgFileUnavailable
            and results[classic_only].message
            == f"no files matching retail using {strategy} strategy"
        )

    else:
        assert False


@pytest.mark.asyncio
async def test_curse_any_flavour_strategy(iw_manager: Manager):
    flavourful = Defn('curse', 'molinari', strategy=Strategy.any_flavour)
    retail_only = Defn('curse', 'mythic-dungeon-tools', strategy=Strategy.any_flavour)
    classic_only = Defn('curse', 'classiccastbars', strategy=Strategy.any_flavour)

    results = await iw_manager.resolve([flavourful, retail_only, classic_only])
    assert all(type(r) is Pkg for r in results.values())


@pytest.mark.asyncio
async def test_curse_version_pinning(iw_manager: Manager):
    defn = Defn('curse', 'molinari').with_version('70300.51-Release')
    results = await iw_manager.resolve([defn])
    assert (
        results[defn].options.strategy == Strategy.version
        and results[defn].version == '70300.51-Release'
    )


@pytest.mark.parametrize(
    'iw_config_values',
    [Flavour.retail],
    indirect=True,
)
@pytest.mark.asyncio
async def test_curse_deps_retrieved(iw_manager: Manager):
    defn = Defn('curse', 'bigwigs-voice-korean')

    results = await iw_manager.resolve([defn], with_deps=True)
    assert {'bigwigs-voice-korean', 'big-wigs'} == {d.slug for d in results.values()}


@pytest.mark.asyncio
async def test_curse_changelog_is_url(iw_manager: Manager):
    molinari = Defn('curse', 'molinari')

    results = await iw_manager.resolve([molinari])
    assert re.match(
        r'https://addons-ecs\.forgesvc\.net/api/v2/addon/\d+/file/\d+/changelog',
        results[molinari].changelog_url,
    )


@pytest.mark.asyncio
async def test_wowi_basic(iw_manager: Manager):
    defn = Defn('wowi', '13188-molinari')
    results = await iw_manager.resolve([defn])
    assert type(results[defn]) is Pkg


@pytest.mark.asyncio
async def test_wowi_changelog_is_data_url(iw_manager: Manager):
    molinari = Defn('wowi', '13188-molinari')
    results = await iw_manager.resolve([molinari])
    assert results[molinari].changelog_url.startswith('data:,')


@pytest.mark.asyncio
async def test_tukui_basic(iw_manager: Manager):
    regular = Defn('tukui', '1')
    ui_suite = Defn('tukui', '-1')

    results = await iw_manager.resolve([regular, ui_suite])

    if iw_manager.config.game_flavour is Flavour.retail:
        assert type(results[regular]) is Pkg and results[regular].name == 'MerathilisUI'
        assert type(results[ui_suite]) is Pkg and results[ui_suite].name == 'Tukui'
    else:
        assert type(results[regular]) is Pkg and results[regular].name == 'Tukui'
        assert type(results[ui_suite]) is R.PkgNonexistent


@pytest.mark.parametrize(
    'iw_config_values',
    [Flavour.retail],
    indirect=True,
)
@pytest.mark.asyncio
async def test_tukui_ui_suite_aliases_for_retail(iw_manager: Manager):
    tukui_id = Defn('tukui', '-1')
    tukui_slug = Defn('tukui', 'tukui')
    elvui_id = Defn('tukui', '-2')
    elvui_slug = Defn('tukui', 'elvui')

    results = await iw_manager.resolve([tukui_id, tukui_slug, elvui_id, elvui_slug])

    assert results[tukui_id].id == results[tukui_slug].id
    assert results[elvui_id].id == results[elvui_slug].id


@pytest.mark.asyncio
async def test_tukui_changelog_url_for_addon_type(iw_manager: Manager):
    ui_suite = Defn('tukui', '-1')
    regular_addon = Defn('tukui', '1')

    results = await iw_manager.resolve([ui_suite, regular_addon])

    if iw_manager.config.game_flavour is Flavour.retail:
        assert results[ui_suite].changelog_url == 'https://www.tukui.org/ui/tukui/changelog#20.23'
    assert results[regular_addon].changelog_url.startswith('data:,')


@pytest.mark.asyncio
async def test_github_basic(iw_manager: Manager):
    release_json = Defn('github', 'nebularg/PackagerTest')
    releaseless = Defn('github', 'AdiAddons/AdiBags')
    nonexistent = Defn('github', 'layday/foobar')

    results = await iw_manager.resolve([release_json, releaseless, nonexistent])

    release_json_result = results[release_json]
    if iw_manager.config.game_flavour is Flavour.burning_crusade_classic:
        assert type(release_json_result) is R.PkgFileUnavailable
    else:
        assert type(release_json_result) is Pkg
        if iw_manager.config.game_flavour is Flavour.vanilla_classic:
            assert 'classic' in release_json_result.download_url
        assert 'nolib' not in release_json_result.download_url
    releaseless_result = results[releaseless]
    assert (
        type(releaseless_result) is R.PkgFileUnavailable
        and releaseless_result.message == 'release not found'
    )
    assert type(results[nonexistent]) is R.PkgNonexistent


@pytest.mark.asyncio
async def test_github_changelog_is_data_url(iw_manager: Manager):
    defn = Defn('github', 'p3lim-wow/Molinari')
    results = await iw_manager.resolve([defn])
    assert results[defn].changelog_url.startswith('data:,')


@pytest.mark.parametrize('resolver', Manager.RESOLVERS)
@pytest.mark.asyncio
async def test_unsupported_strategies(iw_manager: Manager, resolver: Resolver):
    defn = Defn(resolver.source, 'foo')
    for strategy in set(Strategy) - {Strategy.version} - resolver.strategies:
        strategy_defn = evolve_model_obj(defn, strategy=strategy)

        results = await iw_manager.resolve([strategy_defn])

        assert (
            type(results[strategy_defn]) is R.PkgStrategyUnsupported
            and results[strategy_defn].message == f"'{strategy}' strategy is not valid for source"
        )


@pytest.mark.parametrize(
    ('resolver', 'url', 'extracted_alias'),
    [
        (CurseResolver, 'https://www.curseforge.com/wow/addons/molinari', 'molinari'),
        (CurseResolver, 'https://www.curseforge.com/wow/addons/molinari/download', 'molinari'),
        (WowiResolver, 'https://www.wowinterface.com/downloads/landing.php?fileid=13188', '13188'),
        (WowiResolver, 'https://wowinterface.com/downloads/landing.php?fileid=13188', '13188'),
        (WowiResolver, 'https://www.wowinterface.com/downloads/fileinfo.php?id=13188', '13188'),
        (WowiResolver, 'https://wowinterface.com/downloads/fileinfo.php?id=13188', '13188'),
        (WowiResolver, 'https://www.wowinterface.com/downloads/download13188-Molinari', '13188'),
        (WowiResolver, 'https://wowinterface.com/downloads/download13188-Molinari', '13188'),
        (WowiResolver, 'https://www.wowinterface.com/downloads/info13188-Molinari.html', '13188'),
        (WowiResolver, 'https://wowinterface.com/downloads/info13188-Molinari.html', '13188'),
        (WowiResolver, 'https://www.wowinterface.com/downloads/info13188', '13188'),
        (WowiResolver, 'https://wowinterface.com/downloads/info13188', '13188'),
        (TukuiResolver, 'https://www.tukui.org/download.php?ui=tukui', 'tukui'),
        (TukuiResolver, 'https://www.tukui.org/addons.php?id=1', '1'),
        (TukuiResolver, 'https://www.tukui.org/classic-addons.php?id=1', '1'),
        (TukuiResolver, 'https://www.tukui.org/classic-tbc-addons.php?id=1', '1'),
        (
            GithubResolver,
            'https://github.com/AdiAddons/AdiButtonAuras',
            'AdiAddons/AdiButtonAuras',
        ),
        (
            GithubResolver,
            'https://github.com/AdiAddons/AdiButtonAuras/releases',
            'AdiAddons/AdiButtonAuras',
        ),
    ],
)
def test_get_alias_from_url(resolver: Resolver, url: str, extracted_alias: str):
    assert resolver.get_alias_from_url(URL(url)) == extracted_alias



"""
Populate the version table from installed packages.
"""
from alembic import op
from sqlalchemy import String, column, table

# revision identifiers, used by Alembic.
revision = '43aa3610e92a'
down_revision = 'd3f542de5ff4'
branch_labels = None
depends_on = None


def upgrade():
    conn = op.get_bind()

    pkg_version_log = table(
        'pkg_version_log',
        column('pkg_source', String),
        column('pkg_id', String),
        column('version', String),
    )
    pkgs = conn.execute('select source, id, version from pkg').fetchall()
    op.bulk_insert(
        pkg_version_log, [{'pkg_source': s, 'pkg_id': i, 'version': v} for s, i, v in pkgs]
    )


def downgrade():
    op.execute('delete from pkg_version_log')



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src',
 'instawow_gui': 'gui-webview/src/instawow_gui',
 'instawow_gui.frontend': 'gui-webview/src/instawow_gui/frontend',
 'instawow_gui.resources': 'gui-webview/src/instawow_gui/resources',
 'instawow_gui.webview2': 'gui-webview/src/instawow_gui/webview2'}

packages = \
['instawow',
 'instawow.migrations',
 'instawow.migrations.versions',
 'instawow.wa_templates',
 'instawow_gui',
 'instawow_gui.frontend',
 'instawow_gui.resources',
 'instawow_gui.webview2']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4',
 'alembic>=1.7.0',
 'click>=8.0.0',
 'jinja2>=3.0.0',
 'loguru>=0.5.3',
 'pluggy>=0.13',
 'prompt-toolkit>=3.0.15,<4',
 'pydantic>=1.9.0,<2',
 'questionary>=1.10',
 'rapidfuzz>=1.4.1',
 'sqlalchemy>=1.4.23',
 'typing-extensions>=4.0.0',
 'yarl>=1.6.3']

extras_require = \
{'gui': ['aiohttp-rpc>=1.0.0', 'toga>=0.3.0.dev30'],
 'test': ['aresponses>=2.1.4',
          'coverage[toml]>=6.2',
          'pytest>=6.2.5',
          'pytest-asyncio>=0.16.0',
          'pytest-xdist>=2.5.0'],
 'types': ['sqlalchemy2-stubs']}

entry_points = \
{'console_scripts': ['instawow = instawow.cli:main']}

setup_kwargs = {
    'name': 'instawow',
    'version': '1.37.0',
    'description': 'World of Warcraft add-on manager',
    'long_description': "*instawow*\n==========\n\n.. image:: https://img.shields.io/matrix/wow-addon-management:matrix.org\n   :target: https://matrix.to/#/#wow-addon-management:matrix.org?via=matrix.org\n   :alt: Matrix channel\n\n*instawow* is an add-on manager for World of Warcraft.\nIt can be used to install, update and remove add-ons from GitHub,\nCurseForge, WoWInterface and Tukui.\nSome of the features of *instawow* are:\n\n- Interoperable CLI and GUI\n- Fuzzy search with download scoring, backed by a catalogue which\n  combines add-ons from WoWInterface, CurseForge, Tukui and GitHub\n- Support for add-on URLs and *instawow*-specific URNs\n- Add-on reconciliation which works with all three major hosts\n- Rollback – the ability to revert problematic updates\n- Multiple update channels\n- Automatic installation of add-on dependencies\n- Version pinning of CurseForge and GitHub add-ons\n- Wago integration – a WeakAuras Companion clone which can be managed like\n  any other add-on\n\n.. list-table::\n   :widths: 50 50\n\n   * - .. figure:: https://asciinema.org/a/8m36ncAoyTmig4MXfQM8YjE6a.svg\n          :target: https://asciinema.org/a/8m36ncAoyTmig4MXfQM8YjE6a?autoplay=1\n          :alt: Asciicast demonstrating the operation of instawow\n          :width: 640\n     - .. figure:: https://raw.githubusercontent.com/layday/instawow/main/gui-webview/screenshots/v1.34.1.png\n          :target: https://github.com/layday/instawow/releases/latest\n          :alt: instawow-gui main window\n          :width: 640\n\nInstallation\n------------\n\nYou can download pre-built binaries of *instawow* from GitHub:\n\n- `Binaries <https://github.com/layday/instawow/releases/latest>`__\n\nIf you'd prefer to install *instawow* from source, you are able to choose from:\n\n- `pipx <https://github.com/pipxproject/pipx>`__:\n  ``pipx install instawow`` or ``pipx install instawow[gui]`` for the GUI\n- Vanilla pip:\n  ``python -m pip install -U instawow`` or ``python -m pip install -U instawow[gui]`` for the GUI\n\nCLI operation\n-------------\n\ntl;dr\n~~~~~\n\nBegin by running ``instawow reconcile``\nto register previously-installed add-ons with *instawow*\n(or ``instawow reconcile --auto`` to do the same without user input).\nTo install add-ons, you can search for them using the ``search`` command::\n\n    instawow search molinari\n\nIn addition, *instawow* is able to interpret add-on URLs and *instawow*-specific\nURNs of slugs and host IDs.\nAll of the following will install Molinari from CurseForge::\n\n    instawow install https://www.curseforge.com/wow/addons/molinari\n    instawow install curse:molinari\n    instawow install curse:20338\n\nYou can ``update`` add-ons and ``remove`` them just as you'd install them.\nIf ``update`` is invoked without arguments, it will update all of your\ninstalled add-ons.  You can ``list`` add-ons and view detailed information about\nthem using ``list --format detailed``.\nFor ``list`` and similarly non-destructive commands, the source can be omitted\nand the slug can be abbreviated, e.g. ``instawow reveal moli``\nwill open the Molinari add-on folder in your file manager.\n\nAdd-on reconciliation\n~~~~~~~~~~~~~~~~~~~~~\n\nAdd-on reconciliation is not automatic – *instawow* makes a point\nof not automatically assuming ownership of your add-ons.\nHowever, you can automate reconciliation with ``reconcile --auto``\nand *instawow* will prioritise add-ons from CurseForge.\nReconciled add-ons are reinstalled because the installed version cannot be\nextracted reliably.\n\nAdd-on search\n~~~~~~~~~~~~~\n\n*instawow* comes with a rudimentary ``search`` command\nwith results ranked based on edit distance and popularity.\nSearch uses a collated add-on catalogue which is updated\n`once daily <https://github.com/layday/instawow-data/tree/data>`__.\nYou can install multiple add-ons directly from search.\n\nReverting add-on updates\n~~~~~~~~~~~~~~~~~~~~~~~~\n\n*instawow* keeps a log of all versions of an add-on it has previously\ninstalled.\nAdd-on updates can be undone using the ``instawow rollback`` command.\nAdd-ons which have been rolled back are pinned and will not receive updates.\nRollbacks can themselves be undone with ``instawow rollback --undo``,\nwhich will install the latest version of the specified add-on using\nthe ``default`` strategy.\n\nProfiles\n~~~~~~~~\n\n*instawow* supports multiple game versions through the use of profiles.\nAssuming your default profile is configured for retail,\nyou can create a pristine profile for classic by running::\n\n    instawow -p classic configure\n\nYou can create profiles for other versions of the game (e.g. TBC, PTR or beta)\nin the same way.\nPrefix ``-p <profile>`` to *instawow* commands\nto manage each respective profile.\n\nThe ``any_flavour`` strategy can be used to install add-ons from CurseForge\nwhich do not have Classic releases but are known to work just as well::\n\n    instawow -p classic install -s any_flavour https://www.curseforge.com/wow/addons/colorpickerplus\n\nWeakAura updater\n~~~~~~~~~~~~~~~~\n\n*instawow* contains a WeakAura updater modelled after\n`WeakAuras Companion <https://weakauras.wtf/>`__.  To use the updater\nand provided that you have WeakAuras installed::\n\n    instawow weakauras-companion build\n    instawow install instawow:weakauras-companion\n\nYou will have to rebuild the companion add-on prior to updating\nto receive aura updates.  If you would like to check for updates on\nevery invocation of ``instawow update``, install the\n``instawow:weakauras-companion-autoupdate`` variant::\n\n    instawow install instawow:weakauras-companion-autoupdate\n    instawow update\n\nPlug-ins\n~~~~~~~~\n\n*instawow* can be extended using plug-ins.  Plug-ins can be used to add support\nfor arbitrary hosts and add new commands to the CLI.  You will find a sample\nplug-in in ``tests/plugin``.\n\nMetadata sourcing\n-----------------\n\nCurseForge\n~~~~~~~~~~\n\nCurseForge is set to retire its unauthenticated add-on API by the end of Q1 2022.\nCurseForge will be issuing keys for the new API conditionally and which\nadd-on managers are obligated to conceal.\nThe new API is therefore unworkable for add-on managers except through a\nproxy service, which the author of this particular add-on manager cannot afford.\nAt the same time, CurseForge will be providing the option for authors to unlist\ntheir add-ons from the new API, and downloads intitiated through the new API\nwill not count towards author credits for the ad revenue sharing programme.\n\nGitHub\n~~~~~~\n\n*instawow* supports WoW add-ons *released* on GitHub – that is to say that\nthe repository must have a release (tags won't work) and the release must\nhave a ZIP file attached to it as an asset.  In addition, releases must have\na `release.json` file which maps release archives to game versions.\n*instawow* will not install or build add-ons directly from\nsource, or from tarballs or 'zipballs', and will not validate\nthe contents of the ZIP file.\n\nTransparency\n------------\n\nWeb requests initiated by *instawow* can be identified by its user agent string.\n\nEvery 24 hours, on launch, *instawow* will query `PyPI <https://pypi.org>`__ –\nthe canonical Python package index – to check for *instawow* updates.\n\nContributing\n------------\n\nBug reports and fixes are welcome.  Do open an issue before committing to\nmaking any significant changes.\n\nRelated work\n------------\n\nThe author of `strongbox <https://github.com/ogri-la/strongbox>`__ has been\ncataloguing similar software.  If you are unhappy\nwith *instawow*, you might find one of these\n`other <https://ogri-la.github.io/wow-addon-managers/>`__ add-on managers more\nto your liking.\n",
    'author': 'layday',
    'author_email': 'layday@protonmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)



from __future__ import annotations

from collections.abc import Awaitable, Callable
from pathlib import Path

import pytest

from instawow.common import Flavour
from instawow.manager import Manager
from instawow.matchers import (
    AddonFolder,
    FolderAndDefnPairs,
    get_unreconciled_folders,
    match_addon_names_with_folder_names,
    match_folder_name_subsets,
    match_toc_source_ids,
)
from instawow.resolvers import Defn
from instawow.utils import TocReader


def write_addons(iw_manager: Manager, *addons: str):
    for addon in addons:
        (iw_manager.config.addon_dir / addon).mkdir()
        (iw_manager.config.addon_dir / addon / f'{addon}.toc').touch()


@pytest.fixture
def molinari(iw_manager: Manager):
    molinari_folder = iw_manager.config.addon_dir / 'Molinari'
    molinari_folder.mkdir()
    (molinari_folder / 'Molinari.toc').write_text(
        '''\
## X-Curse-Project-ID: 20338
## X-WoWI-ID: 13188
'''
    )
    yield molinari_folder


def test_reconcile_addon_folder_is_comparable_to_str(molinari: Path):
    addon_folder = AddonFolder(molinari.name, TocReader.from_addon_path(molinari))
    assert isinstance(addon_folder, AddonFolder) and addon_folder <= 'Molinari'


def test_reconcile_addon_folder_can_extract_defns_from_toc(molinari: Path):
    addon_folder = AddonFolder(molinari.name, TocReader.from_addon_path(molinari))
    assert addon_folder.defns_from_toc == {Defn('curse', '20338'), Defn('wowi', '13188')}


@pytest.mark.asyncio
async def test_reconcile_invalid_addons_discarded(iw_manager: Manager):
    iw_manager.config.addon_dir.joinpath('foo').mkdir()
    iw_manager.config.addon_dir.joinpath('bar').touch()
    folders = get_unreconciled_folders(iw_manager)
    assert folders == frozenset()
    assert await match_toc_source_ids(iw_manager, folders) == []
    assert await match_folder_name_subsets(iw_manager, folders) == []


@pytest.mark.asyncio
@pytest.mark.parametrize(
    'test_func',
    [match_toc_source_ids, match_folder_name_subsets, match_addon_names_with_folder_names],
)
async def test_reconcile_multiple_defns_per_addon_contained_in_results(
    iw_manager: Manager,
    molinari: Path,
    test_func: Callable[[Manager, frozenset[AddonFolder]], Awaitable[FolderAndDefnPairs]],
):
    ((_, matches),) = await test_func(iw_manager, get_unreconciled_folders(iw_manager))
    expected = {Defn('curse', '20338'), Defn('wowi', '13188')}
    if test_func is match_addon_names_with_folder_names:
        expected.add(Defn('github', 'p3lim-wow/Molinari'))
    assert expected == set(matches)


@pytest.mark.asyncio
async def test_reconcile_results_vary_by_game_flavour(iw_manager: Manager):
    write_addons(iw_manager, 'AdiBags', 'AdiBags_Config')
    ((_, matches),) = await match_folder_name_subsets(
        iw_manager, get_unreconciled_folders(iw_manager)
    )
    if iw_manager.config.game_flavour is Flavour.retail:
        assert {
            Defn('curse', '23350'),
            Defn('curse', '333072'),
            Defn('curse', '431557'),
            Defn('wowi', '26025'),
        } == set(matches)
    elif iw_manager.config.game_flavour in {
        Flavour.vanilla_classic,
        Flavour.burning_crusade_classic,
    }:
        assert {
            Defn('curse', '23350'),
            Defn('curse', '333072'),
            Defn('wowi', '26025'),
        } == set(matches)
    else:
        assert False



"""
Remove ``PkgOptions`` strategy constraint.
"""
from alembic import op
from sqlalchemy import Enum, String

# revision identifiers, used by Alembic.
revision = '2bfcfe824fe0'
down_revision = '58a8306c3a5b'
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table('pkg_options') as batch_op:
        batch_op.alter_column('strategy', type_=String)


def downgrade():
    with op.batch_alter_table('pkg_options') as batch_op:
        batch_op.alter_column('strategy', type_=Enum('default', 'latest'))



"""
Make basenames out of add-on paths.
"""
from pathlib import Path

from alembic import context, op
from sqlalchemy import String, column, table

# revision identifiers, used by Alembic.
revision = '58a8306c3a5b'
down_revision = 'e4ae835a34be'
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table('pkg_folder') as batch_op:
        batch_op.alter_column('path', new_column_name='name')

    pkg_folder = table('pkg_folder', column('name', String))
    conn = op.get_bind()
    for (folder,) in conn.execute(pkg_folder.select()):
        conn.execute(
            pkg_folder.update()
            .where(pkg_folder.c.name == op.inline_literal(folder))
            .values({'name': Path(folder).name})
        )


def downgrade():
    addon_dir = Path(context.get_x_argument(as_dictionary=True).get('addon_dir'))

    with op.batch_alter_table('pkg_folder') as batch_op:
        batch_op.alter_column('name', new_column_name='path')

    pkg_folder = table('pkg_folder', column('path', String))
    conn = op.get_bind()
    for (folder,) in conn.execute(pkg_folder.select()):
        conn.execute(
            pkg_folder.update()
            .where(pkg_folder.c.path == op.inline_literal(folder))
            .values({'path': str(addon_dir / folder)})
        )
